@@ -1,8 +1,12 @@
 """Creates a 4D image and plots it
 """
 
+import numpy as np
+import hyperspy.api as hs
+import matplotlib.pyplot as plt
+
 # Create a 2D image stack with random data
-im = signals.Image(np.random.random((16,16,32,32)))
+im = hs.signals.Signal2D(np.random.random((16, 16, 32, 32)))
 
 # Define the axis properties
 im.axes_manager.signal_axes[0].name = ''
@@ -26,9 +30,7 @@ im.axes_manager.navigation_axes[1].scale = 0.3
 im.axes_manager.navigation_axes[1].offset = 100
 
 # Give a title
-im.mapped_parameters.title = 'Random 2D image stack'
+im.metadata.General.title = 'Random 2D image stack'
 
 im.plot()
-show() # No necessary when running in the HyperSpy's IPython profile
-
-
+plt.show()  # No necessary when running in the HyperSpy's IPython profile
