@@ -1,25 +1,28 @@
-"""Creates a 2D hyperspectrum consisting of two gaussians and plots it.
+"""Creates a 2D hyperspectrum consisting of two Gaussians and plots it.
 
-This example can serve as starting point to test other functionalities on the
+This example can serve as starting point to test other functionality on the
 simulated hyperspectrum.
 
 """
+import numpy as np
+import hyperspy.api as hs
+import matplotlib.pyplot as plt
 
 
 # Create an empty spectrum
-s = signals.Spectrum(np.zeros((32, 32, 1024)))
+s = hs.signals.Signal1D(np.zeros((32, 32, 1024)))
 
 # Generate some simple data: two Gaussians with random centers and area
 
 # First we create a model
-m = create_model(s)
+m = s.create_model()
 
 # Define the first gaussian
-gs1 = components.Gaussian()
+gs1 = hs.model.components1D.Gaussian()
 # Add it to the model
 m.append(gs1)
 
-# Set the sparameters
+# Set the parameters
 gs1.sigma.value = 10
 # Make the center vary in the -5,5 range around 128
 gs1.centre.map['values'][:] = 256 + (np.random.random((32, 32)) - 0.5) * 10
@@ -30,7 +33,7 @@ gs1.A.map['values'][:] = 10000 * np.random.random((32, 32))
 gs1.A.map['is_set'][:] = True
 
 # Second gaussian
-gs2 = components.Gaussian()
+gs2 = hs.model.components1D.Gaussian()
 # Add it to the model
 m.append(gs2)
 
@@ -55,4 +58,4 @@ s_model.add_poissonian_noise()
 # Plot the result
 s_model.plot()
 
-show()
+plt.show()
