@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
 name = 'hyperspy'
 
@@ -25,17 +25,18 @@ name = 'hyperspy'
 # When running setup.py the ".dev" string will be replaced (if possible)
 # by the output of "git describe" if git is available or the git
 # hash if .git is present.
-version = "1.6.4"
+version = "1.7.3"
+__version__ = version
 description = "Multidimensional data analysis toolbox"
 license = 'GPL v3'
 
 authors = {
     'all': ('The HyperSpy developers',), }
 
-url = 'http://hyperspy.org'
+url = 'https://hyperspy.org'
 
-download_url = 'http://www.hyperspy.org'
-documentation_url = 'http://hyperspy.org/hyperspy-doc/current/index.html'
+download_url = 'https://www.hyperspy.org'
+documentation_url = 'https://hyperspy.org/hyperspy-doc/current/index.html'
 
 PROJECT_URLS = {
     'Bug Tracker': 'https://github.com/hyperspy/hyperspy/issues',
@@ -83,6 +84,6 @@ info = """
     H y p e r S p y
     Version %s
 
-    http://www.hyperspy.org
+    https://www.hyperspy.org
 
     """ % version.replace('_', ' ')
