@@ -1,39 +1,39 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
-import os
 import logging
 import math
+import os
+import warnings
 
-import matplotlib.pyplot as plt
 import numpy as np
 import dask.array as da
-import scipy.interpolate
-import scipy as sp
-from scipy.signal import savgol_filter
-from scipy.ndimage.filters import gaussian_filter1d
+from scipy import interpolate
+from scipy.signal import savgol_filter, medfilt
+from scipy.ndimage import gaussian_filter1d
 
 from hyperspy.signal import BaseSignal
 from hyperspy._signals.common_signal1d import CommonSignal1D
-from hyperspy.signal_tools import SpikesRemoval
+from hyperspy.signal_tools import (
+    SpikesRemoval, SpikesRemovalInteractive, SimpleMessage)
 from hyperspy.models.model1d import Model1D
 from hyperspy.misc.lowess_smooth import lowess
-
+from hyperspy.misc.utils import is_binned # remove in v2.0
 
 from hyperspy.defaults_parser import preferences
 from hyperspy.signal_tools import (
@@ -48,12 +48,12 @@ from hyperspy.signal_tools import BackgroundRemoval
 from hyperspy.decorators import interactive_range_selector
 from hyperspy.signal_tools import IntegrateArea, _get_background_estimator
 from hyperspy._signals.lazy import LazySignal
-from hyperspy.docstrings.signal1d import CROP_PARAMETER_DOC
-from hyperspy.docstrings.signal import SHOW_PROGRESSBAR_ARG, PARALLEL_ARG, MAX_WORKERS_ARG
+from hyperspy.docstrings.signal1d import CROP_PARAMETER_DOC, SPIKES_REMOVAL_TOOL_DOCSTRING
+from hyperspy.docstrings.signal import (SHOW_PROGRESSBAR_ARG, PARALLEL_ARG, MAX_WORKERS_ARG,
+                                        SIGNAL_MASK_ARG, NAVIGATION_MASK_ARG)
 from hyperspy.docstrings.plot import (
     BASE_PLOT_DOCSTRING, BASE_PLOT_DOCSTRING_PARAMETERS, PLOT1D_DOCSTRING)
 
-
 _logger = logging.getLogger(__name__)
 
 
@@ -129,7 +129,7 @@ def find_peaks_ohaver(y, x=None, slope_thresh=0., amp_thresh=None,
         amp_thresh = 0.1 * y.max()
     peakgroup = np.round(peakgroup)
     if medfilt_radius:
-        d = np.gradient(scipy.signal.medfilt(y, medfilt_radius))
+        d = np.gradient(medfilt(y, medfilt_radius))
     else:
         d = np.gradient(y)
     n = np.round(peakgroup / 2 + 1)
@@ -174,7 +174,7 @@ def find_peaks_ohaver(y, x=None, slope_thresh=0., amp_thresh=None,
                         # centering and scaling
                         yynz = yy != 0
                         coef = np.polyfit(
-                            xxf[yynz], np.log10(np.abs(yy[yynz])), 2)
+                            xxf[yynz], np.log10(abs(yy[yynz])), 2)
                         c1 = coef[2]
                         c2 = coef[1]
                         c3 = coef[0]
@@ -187,7 +187,7 @@ def find_peaks_ohaver(y, x=None, slope_thresh=0., amp_thresh=None,
                         # of y in the sub-group of points near peak.
                         if peakgroup < 7:
                             height = np.max(yy)
-                            position = xx[np.argmin(np.abs(yy - height))]
+                            position = xx[np.argmin(abs(yy - height))]
                         else:
                             position = - ((stdev * c2 / (2 * c3)) - avg)
                             height = np.exp(c1 - c3 * (c2 / (2 * c3)) ** 2)
@@ -222,7 +222,7 @@ def interpolate1D(number_of_interpolation_points, data):
     ch = len(data)
     old_ax = np.linspace(0, 100, ch)
     new_ax = np.linspace(0, 100, ch * ip - (ip - 1))
-    interpolator = scipy.interpolate.interp1d(old_ax, data)
+    interpolator = interpolate.interp1d(old_ax, data)
     return interpolator(new_ax)
 
 
@@ -253,7 +253,7 @@ def _shift1D(data, **kwargs):
         return data
 
     #This is the interpolant function
-    si = sp.interpolate.interp1d(original_axis, data, bounds_error=False,
+    si = interpolate.interp1d(original_axis, data, bounds_error=False,
                               fill_value=fill_value, kind=kind)
 
     #Evaluate interpolated data at shifted positions
@@ -262,96 +262,112 @@ def _shift1D(data, **kwargs):
 
 class Signal1D(BaseSignal, CommonSignal1D):
 
-    """
-    """
+    """General 1D signal class."""
+
     _signal_dimension = 1
 
     def __init__(self, *args, **kwargs):
+        if kwargs.get('ragged', False):
+            raise ValueError("Signal1D can't be ragged.")
         super().__init__(*args, **kwargs)
-        if self.axes_manager.signal_dimension != 1:
-            self.axes_manager.set_signal_dimension(1)
 
-    def _spikes_diagnosis(self, signal_mask=None,
-                          navigation_mask=None):
-        """Plots a histogram to help in choosing the threshold for
+    def _spikes_diagnosis(
+            self,
+            signal_mask=None,
+            navigation_mask=None,
+            show_plot=False,
+            use_gui=False,
+            **kwargs
+            ):
+
+        self._check_signal_dimension_equals_one()
+        dc = self.data
+        axis = self.axes_manager.signal_axes[0].axis
+        if signal_mask is not None:
+            dc = dc[..., ~signal_mask]
+            axis = axis[~signal_mask]
+        if navigation_mask is not None:
+            dc = dc[~navigation_mask, :]
+        if dc.size == 0:
+            raise ValueError("The data size must be higher than 0.")
+        der = abs(np.gradient(dc, axis, axis=-1))
+        n = ((~navigation_mask).sum() if navigation_mask else
+             self.axes_manager.navigation_size)
+
+        # arbitrary cutoff for number of spectra necessary before histogram
+        # data is compressed by finding maxima of each spectrum
+        tmp = BaseSignal(der) if n < 2000 else BaseSignal(np.ravel(der.max(-1)))
+
+        s_ = tmp.get_histogram(**kwargs)
+        s_.axes_manager[0].name = "Derivative magnitude"
+        s_.metadata.Signal.quantity = "Counts"
+        s_.metadata.General.title = "Spikes Analysis"
+
+        if s_.data.size == 1:
+            message = "The derivative of the data is constant."
+            if use_gui:
+                m = SimpleMessage(text=message)
+                try:
+                    m.gui()
+                except (NotImplementedError, ImportError):
+                    # This is only available for traitsui, in case of ipywidgets
+                    # we show a warning
+                    warnings.warn(message)
+            else:
+                warnings.warn(message)
+        elif show_plot:
+            s_.plot(norm="log")
+
+        return s_
+
+    def spikes_diagnosis(self, signal_mask=None, navigation_mask=None, **kwargs):
+        """
+        Plots a histogram to help in choosing the threshold for
         spikes removal.
 
         Parameters
         ----------
-        signal_mask : boolean array
-            Restricts the operation to the signal locations not marked
-            as True (masked)
-        navigation_mask : boolean array
-            Restricts the operation to the navigation locations not
-            marked as True (masked).
+        %s
+        %s
+        **kwargs : dict
+            Keyword arguments pass to
+            :py:meth:`~hyperspy.signal.signal.BaseSignal.get_histogram`
 
         See also
         --------
         spikes_removal_tool
 
         """
+        self._spikes_diagnosis(
+            signal_mask=signal_mask,
+            navigation_mask=navigation_mask,
+            show_plot=True,
+            use_gui=False,
+            **kwargs
+            )
+
+    spikes_diagnosis.__doc__ %= (SIGNAL_MASK_ARG, NAVIGATION_MASK_ARG)
+
+    def spikes_removal_tool(self, signal_mask=None, navigation_mask=None,
+                            threshold='auto', interactive=True,
+                            display=True, toolkit=None, **kwargs):
         self._check_signal_dimension_equals_one()
-        dc = self.data
-        if signal_mask is not None:
-            dc = dc[..., ~signal_mask]
-        if navigation_mask is not None:
-            dc = dc[~navigation_mask, :]
-        der = np.abs(np.diff(dc, 1, -1))
-        n = ((~navigation_mask).sum() if navigation_mask else
-             self.axes_manager.navigation_size)
+        if interactive:
+            sr = SpikesRemovalInteractive(self,
+                                          signal_mask=signal_mask,
+                                          navigation_mask=navigation_mask,
+                                          threshold=threshold)
+            return sr.gui(display=display, toolkit=toolkit)
+        else:
+            sr = SpikesRemoval(self,
+                               signal_mask=signal_mask,
+                               navigation_mask=navigation_mask,
+                               threshold=threshold, **kwargs)
+            sr.remove_all_spikes()
+            return sr
 
-        # arbitrary cutoff for number of spectra necessary before histogram
-        # data is compressed by finding maxima of each spectrum
-        tmp = BaseSignal(der) if n < 2000 else BaseSignal(
-            np.ravel(der.max(-1)))
-
-        # get histogram signal using smart binning and plot
-        tmph = tmp.get_histogram()
-        tmph.plot()
-
-        # Customize plot appearance
-        plt.gca().set_title('')
-        plt.gca().fill_between(tmph.axes_manager[0].axis,
-                               tmph.data,
-                               facecolor='#fddbc7',
-                               interpolate=True,
-                               color='none')
-        ax = tmph._plot.signal_plot.ax
-        axl = tmph._plot.signal_plot.ax_lines[0]
-        axl.set_line_properties(color='#b2182b')
-        plt.xlabel('Derivative magnitude')
-        plt.ylabel('Log(Counts)')
-        ax.set_yscale('log')
-        ax.set_ylim(10 ** -1, plt.ylim()[1])
-        ax.set_xlim(plt.xlim()[0], 1.1 * plt.xlim()[1])
-        plt.draw()
-
-    def spikes_removal_tool(self, signal_mask=None,
-                            navigation_mask=None, display=True, toolkit=None):
-        """Graphical interface to remove spikes from EELS spectra.
-
-        Parameters
-        ----------
-        signal_mask : boolean array
-            Restricts the operation to the signal locations not marked
-            as True (masked)
-        navigation_mask : boolean array
-            Restricts the operation to the navigation locations not
-            marked as True (masked)
-        %s
-        %s
-
-        See also
-        --------
-        _spikes_diagnosis
-
-        """
-        self._check_signal_dimension_equals_one()
-        sr = SpikesRemoval(self,
-                           navigation_mask=navigation_mask,
-                           signal_mask=signal_mask)
-        return sr.gui(display=display, toolkit=toolkit)
-    spikes_removal_tool.__doc__ %= (DISPLAY_DT, TOOLKIT_DT)
+    spikes_removal_tool.__doc__ = SPIKES_REMOVAL_TOOL_DOCSTRING % (
+        SIGNAL_MASK_ARG, NAVIGATION_MASK_ARG, "", DISPLAY_DT, TOOLKIT_DT,)
 
     def create_model(self, dictionary=None):
         """Create a model for the current data.
@@ -405,6 +421,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
         ------
         SignalDimensionError
             If the signal dimension is not 1.
+        NotImplementedError
+            If the signal axis is a non-uniform axis.
         """
         if not np.any(shift_array):
             # Nothing to do, the shift array if filled with zeros
@@ -414,6 +432,10 @@ class Signal1D(BaseSignal, CommonSignal1D):
         self._check_signal_dimension_equals_one()
         axis = self.axes_manager.signal_axes[0]
 
+        if not axis.is_uniform:
+            raise NotImplementedError(
+                "This operation is not implemented for non-uniform axes.")
+
         # Figure out min/max shifts, and translate to shifts in index as well
         minimum, maximum = np.nanmin(shift_array), np.nanmax(shift_array)
         if minimum < 0:
@@ -504,10 +526,12 @@ class Signal1D(BaseSignal, CommonSignal1D):
         Parameters
         ----------
         start, end : int or float
-            The limits of the interval. If int they are taken as the
-            axis index. If float they are taken as the axis value.
+            The limits of the interval. If int, they are taken as the
+            axis index. If float, they are taken as the axis value.
         delta : int or float
-            The windows around the (start, end) to use for interpolation
+            The windows around the (start, end) to use for interpolation. If
+            int, they are taken as index steps. If float, they are taken in
+            units of the axis value.
         %s
         %s
         %s
@@ -528,22 +552,30 @@ class Signal1D(BaseSignal, CommonSignal1D):
         i1 = axis._get_index(start)
         i2 = axis._get_index(end)
         if isinstance(delta, float):
-            delta = int(delta / axis.scale)
-        i0 = int(np.clip(i1 - delta, 0, np.inf))
-        i3 = int(np.clip(i2 + delta, 0, axis.size))
+            if isinstance(start, int):
+                start = axis.axis[start]
+            if isinstance(end, int):
+                end = axis.axis[end]
+            i0 = axis._get_index(start-delta) if start-delta < axis.low_value else 0
+            i3 = axis._get_index(end+delta) if end+delta > axis.high_value else axis.size
+        else:
+            i0 = int(np.clip(i1 - delta, 0, np.inf))
+            i3 = int(np.clip(i2 + delta, 0, axis.size))
 
         def interpolating_function(dat):
-            dat_int = sp.interpolate.interp1d(
+            dat_int = interpolate.interp1d(
                 list(range(i0, i1)) + list(range(i2, i3)),
                 dat[i0:i1].tolist() + dat[i2:i3].tolist(),
                 **kwargs)
             dat[i1:i2] = dat_int(list(range(i1, i2)))
             return dat
-        self._map_iterate(interpolating_function,
-                          ragged=False,
-                          parallel=parallel,
-                          show_progressbar=show_progressbar,
-                          max_workers=max_workers)
+        self.map(
+            interpolating_function,
+            ragged=False,
+            parallel=parallel,
+            show_progressbar=show_progressbar,
+            max_workers=max_workers,
+        )
         self.events.data_changed.trigger(obj=self)
 
     interpolate_in_between.__doc__ %= (SHOW_PROGRESSBAR_ARG, PARALLEL_ARG, MAX_WORKERS_ARG)
@@ -607,12 +639,17 @@ class Signal1D(BaseSignal, CommonSignal1D):
         ------
         SignalDimensionError
             If the signal dimension is not 1.
+        NotImplementedError
+            If the signal axis is a non-uniform axis.
         """
         if show_progressbar is None:
             show_progressbar = preferences.General.show_progressbar
         self._check_signal_dimension_equals_one()
         ip = number_of_interpolation_points + 1
         axis = self.axes_manager.signal_axes[0]
+        if not axis.is_uniform:
+            raise NotImplementedError(
+                "The function is not implemented for non-uniform signal axes.")
         self._check_navigation_mask(mask)
         # we compute for now
         if isinstance(start, da.Array):
@@ -647,7 +684,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
         if interpolate is True:
             shift_array = shift_array / ip
         shift_array = shift_array * axis.scale
-        if self._lazy:
+        if shift_signal._lazy:
             # We must compute right now because otherwise any changes to the
             # axes_manager of the signal later in the workflow may result in
             # a wrong shift_array
@@ -669,7 +706,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
                 fill_value=np.nan,
                 also_align=None,
                 mask=None,
-                show_progressbar=None):
+                show_progressbar=None,
+                iterpath="flyback"):
         """Estimate the shifts in the signal axis using
         cross-correlation and use the estimation to align the data in place.
         This method can only estimate the shift by comparing
@@ -743,23 +781,24 @@ class Signal1D(BaseSignal, CommonSignal1D):
                             'appropriately), which might take a long time. '
                             'Use expand=False to only pass through the data '
                             'once.')
-        shift_array = self.estimate_shift1D(
-            start=start,
-            end=end,
-            reference_indices=reference_indices,
-            max_shift=max_shift,
-            interpolate=interpolate,
-            number_of_interpolation_points=number_of_interpolation_points,
-            mask=mask,
-            show_progressbar=show_progressbar)
-        signals_to_shift = [self] + also_align
-        for signal in signals_to_shift:
-            signal.shift1D(shift_array=shift_array,
-                           interpolation_method=interpolation_method,
-                           crop=crop,
-                           fill_value=fill_value,
-                           expand=expand,
-                           show_progressbar=show_progressbar)
+        with self.axes_manager.switch_iterpath(iterpath):
+            shift_array = self.estimate_shift1D(
+                start=start,
+                end=end,
+                reference_indices=reference_indices,
+                max_shift=max_shift,
+                interpolate=interpolate,
+                number_of_interpolation_points=number_of_interpolation_points,
+                mask=mask,
+                show_progressbar=show_progressbar)
+            signals_to_shift = [self] + also_align
+            for signal in signals_to_shift:
+                signal.shift1D(shift_array=shift_array,
+                               interpolation_method=interpolation_method,
+                               crop=crop,
+                               fill_value=fill_value,
+                               expand=expand,
+                               show_progressbar=show_progressbar)
     align1D.__doc__ %= (CROP_PARAMETER_DOC, SHOW_PROGRESSBAR_ARG)
 
     def integrate_in_range(self, signal_range='interactive',
@@ -816,9 +855,9 @@ class Signal1D(BaseSignal, CommonSignal1D):
             "instead.")
         deprecation_warning(msg)
 
-        if signal_range == 'interactive':
+        if signal_range == 'interactive':  # pragma: no cover
             self_copy = self.deepcopy()
-            ia = IntegrateArea(self_copy, signal_range)
+            ia = IntegrateArea(self_copy)
             ia.gui(display=display, toolkit=toolkit)
             integrated_signal1D = self_copy
         else:
@@ -854,6 +893,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
         ------
         SignalDimensionError
             If the signal dimension is not 1.
+        NotImplementedError
+            If called with a non-uniform axes.
         """
         self._check_signal_dimension_equals_one()
         calibration = Signal1DCalibration(self)
@@ -893,11 +934,20 @@ class Signal1D(BaseSignal, CommonSignal1D):
         %s
         %s
 
+        Raises
+        ------
+        NotImplementedError
+            If the signal axis is a non-uniform axis.
+
         Notes
         -----
         More information about the filter in `scipy.signal.savgol_filter`.
         """
         self._check_signal_dimension_equals_one()
+        if not self.axes_manager.signal_axes[0].is_uniform:
+            raise NotImplementedError(
+            "This functionality is not implemented for signals with non-uniform axes. "
+            "Consider using `smooth_lowess` instead.")
         if (polynomial_order is not None and
                 window_length is not None):
             axis = self.axes_manager.signal_axes[0]
@@ -997,8 +1047,14 @@ class Signal1D(BaseSignal, CommonSignal1D):
         ------
         SignalDimensionError
             If the signal dimension is not 1.
+        NotImplementedError
+            If the signal axis is a non-uniform axis.
         """
         self._check_signal_dimension_equals_one()
+        if not self.axes_manager.signal_axes[0].is_uniform:
+            raise NotImplementedError(
+            "This functionality is not implemented for signals with non-uniform axes. "
+            "Consider using `smooth_lowess` instead.")
         if smoothing_parameter is None:
             smoother = SmoothingTV(self)
             return smoother.gui(display=display, toolkit=toolkit)
@@ -1027,7 +1083,13 @@ class Signal1D(BaseSignal, CommonSignal1D):
         ------
         SignalDimensionError
             If the signal dimension is not 1.
+        NotImplementedError
+            If the signal axis is a non-uniform axis.
         """
+        if not self.axes_manager.signal_axes[0].is_uniform:
+            raise NotImplementedError(
+            "This functionality is not implemented for signals with non-uniform axes. "
+            "Consider using `smooth_lowess` instead.")
         self._check_signal_dimension_equals_one()
         smoother = ButterworthFilter(self)
         if cutoff_frequency_ratio is not None:
@@ -1067,7 +1129,15 @@ class Signal1D(BaseSignal, CommonSignal1D):
         else:
             try:
                 axis = self.axes_manager.signal_axes[0]
-                scale_factor = axis.scale if self.metadata.Signal.binned else 1
+                if is_binned(self):
+                # in v2 replace by
+                # if axis.is_binned:
+                    if axis.is_uniform:
+                        scale_factor = axis.scale
+                    else:
+                        scale_factor = np.gradient(axis.axis)
+                else:
+                    scale_factor = 1
                 bkg = background_estimator.function_nd(axis.axis) * scale_factor
                 result = self - bkg
             except MemoryError:
@@ -1263,11 +1333,16 @@ class Signal1D(BaseSignal, CommonSignal1D):
         ------
         ValueError
             If FWHM is equal or less than zero.
-
         SignalDimensionError
             If the signal dimension is not 1.
+        NotImplementedError
+            If the signal axis is a non-uniform axis.
         """
         self._check_signal_dimension_equals_one()
+        for _axis in self.axes_manager.signal_axes:
+            if not _axis.is_uniform:
+                raise NotImplementedError(
+                    "The function is not implemented for non-uniform axes.")
         if FWHM <= 0:
             raise ValueError(
                 "FWHM must be greater than zero")
@@ -1426,7 +1501,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
                          ragged=True,
                          parallel=parallel,
                          max_workers=max_workers,
-                         inplace=False)
+                         inplace=False,
+                         lazy_output=False)
         return peaks.data
 
     find_peaks1D_ohaver.__doc__ %= (PARALLEL_ARG, MAX_WORKERS_ARG)
@@ -1494,23 +1570,24 @@ class Signal1D(BaseSignal, CommonSignal1D):
             )
             parallel = False
 
-        axis = self.axes_manager.signal_axes[0]
-        # x = axis.axis
+        # axis is a keyword already used by self.map so calling this axis_arg
+        # to avoid "parameter collision
+        axis_arg = self.axes_manager.signal_axes[0]
         maxval = self.axes_manager.navigation_size
         show_progressbar = show_progressbar and maxval > 0
 
         def estimating_function(spectrum,
                                 window=None,
                                 factor=0.5,
-                                axis=None):
-            x = axis.axis
+                                axis_arg=None):
+            x = axis_arg.axis
             if window is not None:
-                vmax = axis.index2value(spectrum.argmax())
-                slices = axis._get_array_slices(
+                vmax = axis_arg.index2value(spectrum.argmax())
+                slices = axis_arg._get_array_slices(
                     slice(vmax - window * 0.5, vmax + window * 0.5))
                 spectrum = spectrum[slices]
                 x = x[slices]
-            spline = scipy.interpolate.UnivariateSpline(
+            spline = interpolate.UnivariateSpline(
                 x,
                 spectrum - factor * spectrum.max(),
                 s=0)
@@ -1520,15 +1597,17 @@ class Signal1D(BaseSignal, CommonSignal1D):
             else:
                 return np.full((2,), np.nan)
 
-        both = self._map_iterate(estimating_function,
-                                 window=window,
-                                 factor=factor,
-                                 axis=axis,
-                                 ragged=False,
-                                 inplace=False,
-                                 parallel=parallel,
-                                 show_progressbar=show_progressbar,
-                                 max_workers=None)
+        both = self.map(
+            estimating_function,
+            window=window,
+            factor=factor,
+            axis_arg=axis_arg,
+            ragged=False,
+            inplace=False,
+            parallel=parallel,
+            show_progressbar=show_progressbar,
+            max_workers=max_workers,
+        )
         left, right = both.T.split()
         width = right - left
         if factor == 0.5:
@@ -1551,7 +1630,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
                 self.metadata.General.title +
                 " full-width at %.1f maximum right position" % factor)
         for signal in (left, width, right):
-            signal.axes_manager.set_signal_dimension(0)
+            signal = signal.transpose(signal_axes=[])
             signal.set_signal_type("")
         if return_interval is True:
             return [width, left, right]
@@ -1589,10 +1668,4 @@ class Signal1D(BaseSignal, CommonSignal1D):
 
 class LazySignal1D(LazySignal, Signal1D):
 
-    """
-    """
     _lazy = True
-
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.axes_manager.set_signal_dimension(1)
