@@ -107,10 +107,12 @@ class DigitalMicrographReader(object):
         else:
             self.endian = 'big'
 
-    def parse_tags(self, ntags, group_name='root', group_dict={}):
+    def parse_tags(self, ntags, group_name='root', group_dict=None):
         """Parse the DM file into a dictionary.
 
         """
+        if group_dict is None: #pragma: no cover
+            group_dict = {}    # currently this default value is not used in this script
         unnammed_data_tags = 0
         unnammed_group_tags = 0
         for tag in range(ntags):
@@ -729,7 +731,9 @@ class ImageObject(object):
                     zip(self.names, self.shape, self.scales, self.offsets,
                         self.units))]
 
-    def get_metadata(self, metadata={}):
+    def get_metadata(self, metadata=None):
+        if metadata is None:
+            metadata = {}
         if "General" not in metadata:
             metadata['General'] = {}
         if "Signal" not in metadata:
