@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
 import logging
 import warnings
@@ -33,7 +33,7 @@ _logger = logging.getLogger(__name__)
 
 def _soft_thresh(X, lambda1):
     """Soft-thresholding of array X."""
-    res = np.abs(X) - lambda1
+    res = abs(X) - lambda1
     np.maximum(res, 0.0, out=res)
     res *= np.sign(X)
     return res
@@ -224,7 +224,7 @@ class ORPCA:
         via Stochastic Optimization", Advances in Neural Information Processing
         Systems 26, (2013), pp. 404-412.
     .. [Ruder2016] Sebastian Ruder, "An overview of gradient descent optimization
-        algorithms", arXiv:1609.04747, (2016), http://arxiv.org/abs/1609.04747.
+        algorithms", arXiv:1609.04747, (2016), https://arxiv.org/abs/1609.04747.
 
     """
 
