@@ -1,23 +1,23 @@
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>
 
 
 from collections import OrderedDict
-import math as math
+import math
 import logging
 
 import dask.array as da
@@ -26,6 +26,7 @@ from numba import njit
 
 
 from hyperspy.misc.math_tools import anyfloatin
+from hyperspy.docstrings.utils import REBIN_ARGS
 
 
 _logger = logging.getLogger(__name__)
@@ -80,8 +81,7 @@ def homogenize_ndim(*args):
 
     max_len = max([len(ary.shape) for ary in args])
 
-    return [ary.reshape((1,) * (max_len - len(ary.shape)) + ary.shape)
-            for ary in args]
+    return [ary.reshape((1,) * (max_len - len(ary.shape)) + ary.shape) for ary in args]
 
 
 def _requires_linear_rebin(arr, scale):
@@ -94,44 +94,19 @@ def _requires_linear_rebin(arr, scale):
         rebinning factors
     """
 
-    return (np.asarray(arr.shape) %
-            np.asarray(scale)).any() or anyfloatin(scale)
+    return (np.asarray(arr.shape) % np.asarray(scale)).any() or anyfloatin(scale)
 
 
-def rebin(a, new_shape=None, scale=None, crop=True):
-    """Rebin array.
-
-    rebin ndarray data into a smaller or larger array based on a linear
-    interpolation. Specify either a new_shape or a scale. Scale of 1== no
+def rebin(a, new_shape=None, scale=None, crop=True, dtype=None):
+    """Rebin data into a smaller or larger array based on a linear
+    interpolation. Specify either a new_shape or a scale. Scale of 1 means no
     binning. Scale less than one results in up-sampling.
 
     Parameters
     ----------
     a : numpy array
-    new_shape : a list of floats or integer, default None
-        For each dimension specify the new_shape of the np.array. This will
-        then be converted into a scale.
-    scale : a list of floats or integer, default None
-        For each dimension specify the new:old pixel ratio, e.g. a ratio of 1
-        is no binning and a ratio of 2 means that each pixel in the new
-        spectrum is twice the size of the pixels in the old spectrum.
-        The length of the list should match the dimension of the numpy array.
-        ***Note : Only one of scale or new_shape should be specified otherwise
-        the function will not run***
-    crop: bool, default True
-        When binning by a non-integer number of pixels it is likely that
-        the final row in each dimension contains less than the full quota to
-        fill one pixel.
-
-        e.g. 5*5 array binned by 2.1 will produce two rows containing 2.1
-        pixels and one row containing only 0.8 pixels worth. Selection of
-        crop='True' or crop='False' determines whether or not this
-        'black' line is cropped from the final binned array or not.
-
-        *Please note that if crop=False is used, the final row in each
-        dimension may appear black, if a fractional number of pixels are left
-        over. It can be removed but has been left to preserve total counts
-        before and after binning.*
+        The array to rebin.
+    %s
 
     Returns
     -------
@@ -144,10 +119,10 @@ def rebin(a, new_shape=None, scale=None, crop=True):
 
     Notes
     -----
-    Fast re_bin function Adapted from scipy cookbook
+    Fast ``re_bin`` function Adapted from scipy cookbook
     If rebin function fails with error stating that the function is 'not binned
-    and therefore cannot be rebinned', add binned to metadata with:
-    >>> s.metadata.Signal.binned = True
+    and therefore cannot be rebinned', add binned to axes parameters with:
+    >>> s.axes_manager[axis].is_binned = True
 
     """
     # Series of if statements to check that only one out of new_shape or scale
@@ -161,18 +136,19 @@ def rebin(a, new_shape=None, scale=None, crop=True):
         scale = []
         for i, _ in enumerate(a.shape):
             scale.append(a.shape[i] / new_shape[i])
-    else:
-        new_shape = new_shape
-        scale = scale
+    if isinstance(dtype, str) and dtype != 'same':
+        raise ValueError(
+            '`dtype` argument needs to be None, a numpy dtype or '
+            'the string "same".'
+            )
+
     # check whether or not interpolation is needed.
     if _requires_linear_rebin(arr=a, scale=scale):
         _logger.debug("Using linear_bin")
-        if np.issubdtype(a.dtype, np.integer):
-            # The _linear_bin function below requires a float dtype
-            # because of the default numpy casting rule ('same_kind').
-            a = a.astype("float", casting="safe", copy=False)
-        return _linear_bin(a, scale, crop)
+        return _linear_bin(a, scale, crop, dtype=dtype)
     else:
+        if dtype == 'same':
+            dtype = a.dtype
         _logger.debug("Using standard rebin with lazy support")
         # if interpolation is not needed run fast re_bin function.
         # Adapted from scipy cookbook.
@@ -191,19 +167,19 @@ def rebin(a, new_shape=None, scale=None, crop=True):
         scale = np.asarray(a.shape) // np.asarray(new_shape)
         if scale.max() < 2:
             return a.copy()
+
         if isinstance(a, np.ndarray):
             # most of the operations will fall here and dask is not imported
             rshape = ()
             for athing in zip(new_shape, scale):
                 rshape += athing
             return a.reshape(rshape).sum(axis=tuple(
-                2 * i + 1 for i in range(lenShape)))
+                2 * i + 1 for i in range(lenShape)), dtype=dtype)
         else:
-            import dask.array as da
-
             try:
-                return da.coarsen(np.sum, a, {i: int(f)
-                                              for i, f in enumerate(scale)})
+                return da.coarsen(np.sum, a,
+                                  {i: int(f) for i, f in enumerate(scale)},
+                                  dtype=dtype)
             # we provide slightly better error message in hyperspy context
             except ValueError:
                 raise ValueError(
@@ -211,6 +187,9 @@ def rebin(a, new_shape=None, scale=None, crop=True):
                     "Rebin fewer dimensions at a time to avoid this error"
                 )
 
+# Replacing space is necessary to get the correct indentation
+rebin.__doc__ %= REBIN_ARGS.replace("        ", "    ")
+
 
 @njit(cache=True)
 def _linear_bin_loop(result, data, scale):  # pragma: no cover
@@ -218,7 +197,7 @@ def _linear_bin_loop(result, data, scale):  # pragma: no cover
         # Begin by determining the upper and lower limits of a given new pixel.
         x1 = j * scale
         x2 = min((1 + j) * scale, data.shape[0])
-        value = result[j:j + 1]
+        value = result[j : j + 1]
 
         if (x2 - x1) >= 1:
             # When binning, the first part is to deal with the fractional pixel
@@ -254,7 +233,7 @@ def _linear_bin_loop(result, data, scale):  # pragma: no cover
                 value += data[math.floor(x1)] * (x2 - x1)
 
 
-def _linear_bin(dat, scale, crop=True):
+def _linear_bin(dat, scale, crop=True, dtype=None):
     """Binning of the spectrum image by a non-integer pixel value.
 
     Parameters
@@ -291,6 +270,30 @@ def _linear_bin(dat, scale, crop=True):
             "dimensions specifically, simply set the value in shape to 1."
         )
 
+    # Unsuported dtype value argument
+    dtype_str_same_integer = (isinstance(dtype, str) and dtype == 'same' and
+                              np.issubdtype(dat.dtype, np.integer))
+    dtype_interger = (not isinstance(dtype, str) and
+                      np.issubdtype(dtype, np.integer))
+
+    if dtype_str_same_integer or dtype_interger:
+        raise ValueError(
+            "Linear interpolation requires float dtype, change the "
+            "dtype argument."
+            )
+
+    if np.issubdtype(dat.dtype, np.integer):
+        # The _linear_bin function below requires a float dtype
+        # because of the default numpy casting rule ('same_kind').
+        dtype = float
+    # Make sure that native endian is used as required by numba.jit
+    elif not dat.dtype.isnative:
+        dtype = dat.dtype.type
+
+    # If dtype is not None, it means that we need to change it
+    if dtype is not None:
+        dat = dat.astype(dtype, casting="safe", copy=False)
+
     for axis, s in enumerate(scale):
         # For each iteration of linear_bin the axis being interated over has to
         # be switched to axis[0] in order to carry out the interation loop.
@@ -301,8 +304,7 @@ def _linear_bin(dat, scale, crop=True):
         if not np.issubdtype(s, np.floating):
             s = float(s)
 
-        dim = (math.floor(dat.shape[0] / s) if crop
-               else math.ceil(dat.shape[0] / s))
+        dim = math.floor(dat.shape[0] / s) if crop else math.ceil(dat.shape[0] / s)
         # check function wont bin to zero.
         if dim == 0:
             raise ValueError(
@@ -310,12 +312,14 @@ def _linear_bin(dat, scale, crop=True):
                 "Re-adjust your scale values or run code with "
                 "crop=False to avoid this error."
             )
+
         # Make sure that native endian is used
-        if not dat.dtype.isnative:
-            dat = dat.astype(dat.dtype.type)
+        if dtype is None:
+            dtype = dat.dtype.type
         # Set up the result np.array to have a new axis[0] size for after
         # cropping.
-        result = np.zeros((dim,) + dat.shape[1:], dtype=dat.dtype)
+        result = np.zeros((dim,) + dat.shape[1:], dtype=dtype)
+
         # Carry out binning over axis[0]
         _linear_bin_loop(result=result, data=dat, scale=s)
         # Swap axis[0] back to the original axis location.
@@ -438,7 +442,7 @@ def get_signal_chunk_slice(index, chunks):
     if not isinstance(index, (list, tuple)):
         index = tuple(index)
 
-    chunk_slice_raw_list = da.core.slices_from_chunks(chunks[-len(index):])
+    chunk_slice_raw_list = da.core.slices_from_chunks(chunks[-len(index) :])
     chunk_slice_list = []
     for chunk_slice_raw in chunk_slice_raw_list:
         chunk_slice_list.append(list(chunk_slice_raw)[::-1])
@@ -451,3 +455,132 @@ def get_signal_chunk_slice(index, chunks):
             elif _slice[1].start <= index[1] < _slice[1].stop:
                 return chunk_slice
     raise ValueError("Index out of signal range.")
+
+
+@njit(cache=True)
+def numba_closest_index_round(axis_array, value_array):
+    """For each value in value_array, find the closest value in axis_array and
+    return the result as a numpy array of the same shape as value_array.
+    Use round half towards zero strategy for rounding float to interger.
+
+    Parameters
+    ----------
+    axis_array : numpy array
+    value_array : numpy array
+
+    Returns
+    -------
+    numpy array
+
+    """
+    # initialise the index same dimension as input, force type to int
+    index_array = np.empty_like(value_array, dtype="uint")
+    # assign on flat, iterate on flat.
+    rtol = 1e-12
+    machineepsilon = np.min(np.abs(np.diff(axis_array))) * rtol
+    for i, v in enumerate(value_array.flat):
+        index_array.flat[i] = np.abs(axis_array - v + np.sign(v) * machineepsilon).argmin()
+    return index_array
+
+
+@njit(cache=True)
+def numba_closest_index_floor(axis_array, value_array):  # pragma: no cover
+    """For each value in value_array, find the closest smaller value in
+    axis_array and return the result as a numpy array of the same shape
+    as value_array.
+
+    Parameters
+    ----------
+    axis_array : numpy array
+    value_array : numpy array
+
+    Returns
+    -------
+    numpy array
+
+    """
+    # initialise the index same dimension as input, force type to int
+    index_array = np.empty_like(value_array, dtype="uint")
+    # assign on flat, iterate on flat.
+    for i, v in enumerate(value_array.flat):
+        x = axis_array - v
+        index_array.flat[i] = np.where(x > 0, -np.inf, x).argmax()
+
+    return index_array
+
+
+@njit(cache=True)
+def numba_closest_index_ceil(axis_array, value_array):  # pragma: no cover
+    """For each value in value_array, find the closest larger value in
+    axis_array and return the result as a numpy array of the same shape
+    as value_array.
+
+    Parameters
+    ----------
+    axis_array : numpy array
+    value_array : numpy array
+
+    Returns
+    -------
+    numpy array
+    """
+    # initialise the index same dimension as input, force type to int
+    index_array = np.empty_like(value_array, dtype="uint")
+    # assign on flat, iterate on flat.
+    for i, v in enumerate(value_array.flat):
+        x = axis_array - v
+        index_array.flat[i] = np.where(x < 0, +np.inf, x).argmin()
+
+    return index_array
+
+
+@njit(cache=True)
+def round_half_towards_zero(array, decimals=0):  # pragma: no cover
+    """
+    Round input array using "half towards zero" strategy.
+
+    Parameters
+    ----------
+    array : ndarray
+        Input array.
+
+    decimals : int, optional
+        Number of decimal places to round to (default: 0).
+
+    Returns
+    -------
+    rounded_array : ndarray
+        An array of the same type as a, containing the rounded values.
+    """
+    multiplier = 10 ** decimals
+
+    return np.where(array >= 0,
+                    np.ceil(array * multiplier - 0.5) / multiplier,
+                    np.floor(array * multiplier + 0.5) / multiplier
+                    )
+
+
+@njit(cache=True)
+def round_half_away_from_zero(array, decimals=0):  # pragma: no cover
+    """
+    Round input array using "half away from zero" strategy.
+
+    Parameters
+    ----------
+    array : ndarray
+        Input array.
+
+    decimals : int, optional
+        Number of decimal places to round to (default: 0).
+
+    Returns
+    -------
+    rounded_array : ndarray
+        An array of the same type as a, containing the rounded values.
+    """
+    multiplier = 10 ** decimals
+
+    return np.where(array >= 0,
+                    np.floor(array * multiplier + 0.5) / multiplier,
+                    np.ceil(array * multiplier - 0.5) / multiplier
+                    )
