@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
 """
 The Signal class and its specialized subclasses:
@@ -56,15 +56,22 @@ The Signal class and its specialized subclasses:
         ComplexSignal2D can be reconstructed from them.
 """
 
-# -*- coding: utf-8 -*-
-from hyperspy.extensions import EXTENSIONS as _EXTENSIONS
+from hyperspy.extensions import EXTENSIONS as EXTENSIONS_
 import importlib
 
-_g = globals()
-for _signal, _specs in _EXTENSIONS["signals"].items():
-    if not _specs["lazy"]:
-        _g[_signal] = getattr(
-            importlib.import_module(
-                _specs["module"]), _signal)
 
-del importlib
+__all__ = [
+    signal_ for signal_, specs_ in EXTENSIONS_["signals"].items()
+    if not specs_["lazy"]
+    ]
+
+
+def __dir__():
+    return sorted(__all__)
+
+
+def __getattr__(name):
+    if name in __all__:
+        spec = EXTENSIONS_["signals"][name]
+        return getattr(importlib.import_module(spec['module']), name)
+    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
