@@ -1,4 +1,4 @@
-# Copyright 2007-2012 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
 # This file is part of HyperSpy.
 #
@@ -13,22 +13,22 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with HyperSpy. If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
 
 import numpy as np
 
-from nose.tools import assert_true
-from hyperspy._signals.spectrum import Spectrum
-from hyperspy.hspy import create_model
-from hyperspy.components import Gaussian
+from hyperspy._signals.signal1d import Signal1D
+from hyperspy.components1d import Gaussian
+from hyperspy.decorators import lazifyTestClass
 
 
+@lazifyTestClass
 class TestChiSquared:
 
-    def setUp(self):
-        s = Spectrum(np.array([1.0, 2, 4, 7, 12, 7, 4, 2, 1]))
-        m = create_model(s)
+    def setup_method(self, method):
+        s = Signal1D(np.array([1.0, 2, 4, 7, 12, 7, 4, 2, 1]))
+        m = s.create_model()
         self.model = m
         self.A = 38.022476979172588
         self.sigma = 1.4764966133859543
@@ -39,7 +39,7 @@ class TestChiSquared:
         g = Gaussian()
         m.append(g)
         m.fit()
-        assert_true(np.allclose(m.chisq(), 7.78966223))
+        np.testing.assert_allclose(m.chisq(), 7.78966223)
 
     def test_dof_with_fit(self):
         m = self.model
@@ -48,14 +48,14 @@ class TestChiSquared:
         m.extend((g, g1))
         g1.set_parameters_not_free('A')
         m.fit()
-        assert_true(np.equal(m.dof(), 5))
+        assert np.equal(m.dof(), 5)
 
     def test_red_chisq_with_fit(self):
         m = self.model
         g = Gaussian()
         m.append(g)
         m.fit()
-        assert_true(np.allclose(m.red_chisq(), 1.55793245))
+        np.testing.assert_allclose(m.red_chisq(), 1.55793245)
 
     def test_chisq(self):
         m = self.model
@@ -65,7 +65,7 @@ class TestChiSquared:
         g.centre.value = self.centre
         m.append(g)
         m._calculate_chisq()
-        assert_true(np.allclose(m.chisq(), 7.78966223))
+        np.testing.assert_allclose(m.chisq(), 7.78966223)
 
     def test_dof_with_p0(self):
         m = self.model
@@ -75,7 +75,7 @@ class TestChiSquared:
         g1.set_parameters_not_free('A')
         m._set_p0()
         m._set_current_degrees_of_freedom()
-        assert_true(np.equal(m.dof(), 5))
+        assert np.equal(m.dof(), 5)
 
     def test_red_chisq(self):
         m = self.model
@@ -87,7 +87,7 @@ class TestChiSquared:
         m._set_p0()
         m._set_current_degrees_of_freedom()
         m._calculate_chisq()
-        assert_true(np.allclose(m.red_chisq(), 1.55793245))
+        np.testing.assert_allclose(m.red_chisq(), 1.55793245)
 
     def test_chisq_in_range(self):
         m = self.model
@@ -95,7 +95,7 @@ class TestChiSquared:
         m.append(g)
         m.set_signal_range(1, 7)
         m.fit()
-        assert_true(np.allclose(m.red_chisq(), 2.87544335))
+        np.testing.assert_allclose(m.red_chisq(), 2.20961562)
 
     def test_chisq_with_inactive_components(self):
         m = self.model
@@ -105,7 +105,7 @@ class TestChiSquared:
         m.append(gin)
         gin.active = False
         m.fit()
-        assert_true(np.allclose(m.chisq(), 7.78966223))
+        np.testing.assert_allclose(m.chisq(), 7.78966223)
 
     def test_dof_with_inactive_components(self):
         m = self.model
@@ -115,4 +115,4 @@ class TestChiSquared:
         m.append(gin)
         gin.active = False
         m.fit()
-        assert_true(np.equal(m.dof(), 3))
+        assert np.equal(m.dof(), 3)
