@@ -1,19 +1,19 @@
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
 
 import numpy as np
@@ -202,6 +202,19 @@ class Test_quantification:
             TEM_md.probe_area, 1.2)
         np.testing.assert_approx_equal(TEM_md.Detector.EDS.real_time, 3.1)
 
+    def test_quant_one_intensity_error(self):
+        s = self.signal
+        method = 'CL'
+        kfactors = [1, 2.0009344042484134]
+        intensities = s.get_lines_intensity()[:1]
+        assert len(intensities) == 1
+        with pytest.raises(ValueError):
+            _ = s.quantification(intensities, method, kfactors)
+
+        intensities = s.get_lines_intensity()[0]
+        with pytest.raises(ValueError):
+            _ = s.quantification(intensities, method, kfactors)
+
     def test_quant_lorimer(self):
         s = self.signal
         method = 'CL'
@@ -216,16 +229,25 @@ class Test_quantification:
         np.testing.assert_allclose(res[0].data, np.ones((2, 2)) * 22.70779,
             atol=1e-3)
 
-    def test_quant_lorimer_nav1(self):
+        # Test plot_results
+        s2 = s.inav[0, 0]
+        s2.quantification(intensities, method, kfactors,
+                          composition_units, plot_result=True)
+        np.testing.assert_allclose(res[0].data,  22.70779)
+
+    def test_quant_lorimer_mask(self):
         s = self.signal
-        s = s.inav[0, 0]
         method = 'CL'
         kfactors = [1, 2.0009344042484134]
         composition_units = 'weight'
         intensities = s.get_lines_intensity()
+        mask = np.array([[1, 1], [0, 0]])
         res = s.quantification(intensities, method, kfactors,
-                               composition_units, plot_result=True)
-        np.testing.assert_allclose(res[0].data,  22.70779)
+                               composition_units,
+                               navigation_mask=mask)
+        np.testing.assert_allclose(res[0].data, np.array([
+            [0, 0],
+            [22.70779, 22.70779]]), atol=1e-3)
 
     def test_quant_lorimer_warning(self):
         s = self.signal
@@ -274,7 +296,6 @@ class Test_quantification:
                                    atol=1e-3)
         np.testing.assert_allclose(res[0].data, res4[0][0].data, atol=1e-5)
 
-
     def test_quant_zeta(self):
         s = self.signal
         method = 'zeta'
@@ -479,7 +500,7 @@ class Test_quantification:
 
 
 @lazifyTestClass
-class Test_vacum_mask:
+class Test_vacuum_mask:
 
     def setup_method(self, method):
         s = EDSTEMSpectrum(np.array([np.linspace(0.001, 0.5, 20)] * 100).T)
@@ -491,6 +512,12 @@ class Test_vacum_mask:
         assert s.vacuum_mask().data[0]
         assert not s.vacuum_mask().data[-1]
 
+    def test_vacuum_mask_navigation_dimension_0(self):
+        s = self.signal
+        s2 = s.sum()
+        with pytest.raises(RuntimeError):
+            s2.vacuum_mask()
+
 
 @pytest.mark.parametrize('normalise_poissonian_noise', [True, False])
 def test_decomposition(normalise_poissonian_noise):
@@ -558,6 +585,7 @@ class Test_eds_markers:
 
     def test_manual_add_line(self):
         s = self.signal
+        s.plot()
         s.add_xray_lines_markers(['Zn_La'])
         assert (
             list(s._xray_markers.keys()) ==
@@ -566,8 +594,14 @@ class Test_eds_markers:
         # Check that the line has both a vertical line marker and text marker:
         assert len(s._xray_markers['Zn_La']) == 2
 
+    def test_manual_add_line_error(self):
+        s = self.signal
+        with pytest.raises(RuntimeError):
+            s.add_xray_lines_markers(['Zn_La'])
+
     def test_manual_remove_element(self):
         s = self.signal
+        s.plot()
         s.add_xray_lines_markers(['Zn_Ka', 'Zn_Kb', 'Zn_La'])
         s.remove_xray_lines_markers(['Zn_Kb'])
         assert sorted(s._xray_markers.keys()) == ['Zn_Ka', 'Zn_La']
