@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
 # This file is part of HyperSpy.
 #
@@ -10,13 +10,35 @@
 #
 # HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy.  If not, see <https://www.gnu.org/licenses/#GPL>.
+
 import hyperspy.roi
-from hyperspy.roi import (CircleROI, Line2DROI, Point1DROI, Point2DROI,
-                          RectangularROI, SpanROI)
+from hyperspy.roi import (
+    CircleROI,
+    Line2DROI,
+    Point1DROI,
+    Point2DROI,
+    RectangularROI,
+    SpanROI
+    )
+
 
 __doc__ = hyperspy.roi.__doc__
+
+
+__all__ = [
+    'CircleROI',
+    'Line2DROI',
+    'Point1DROI',
+    'Point2DROI',
+    'RectangularROI',
+    'SpanROI',
+    ]
+
+
+def __dir__():
+    return sorted(__all__)
